/*
 * Copyright (C) 2008 Zeeshan Ali <zeenix@gmail.com>.
 * Copyright (C) 2008 Nokia Corporation, all rights reserved.
 *
 * Author: Zeeshan Ali <zeenix@gmail.com>
 *
 * This file is part of Rygel.
 *
 * Rygel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Rygel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "rygel-tracker-video-item.h"
#include <rygel-media-object.h>
#include <rygel-media-item.h>
#include <libgupnp-av/gupnp-av.h>


#define RYGEL_TRACKER_VIDEO_ITEM_TYPE_METADATA (rygel_tracker_video_item_metadata_get_type ())

typedef enum  {
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE,
	RYGEL_TRACKER_VIDEO_ITEM_METADATA_LAST_KEY
} RygelTrackerVideoItemMetadata;



enum  {
	RYGEL_TRACKER_VIDEO_ITEM_DUMMY_PROPERTY
};
GType rygel_tracker_video_item_metadata_get_type (void);
char** _dynamic_Get3 (DBusGProxy* self, const char* param1, const char* param2, char** param3, int param3_length1, int* result_length1, GError** error);
static void rygel_tracker_video_item_real_fetch_metadata (RygelTrackerItem* base, GError** error);
static gpointer rygel_tracker_video_item_parent_class = NULL;
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType rygel_tracker_video_item_metadata_get_type (void) {
	static GType rygel_tracker_video_item_metadata_type_id = 0;
	if (G_UNLIKELY (rygel_tracker_video_item_metadata_type_id == 0)) {
		static const GEnumValue values[] = {{RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME", "file-name"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME", "mime"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE", "size"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE", "title"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR", "author"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH", "width"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT", "height"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE", "date"}, {RYGEL_TRACKER_VIDEO_ITEM_METADATA_LAST_KEY, "RYGEL_TRACKER_VIDEO_ITEM_METADATA_LAST_KEY", "last-key"}, {0, NULL, NULL}};
		rygel_tracker_video_item_metadata_type_id = g_enum_register_static ("RygelTrackerVideoItemMetadata", values);
	}
	return rygel_tracker_video_item_metadata_type_id;
}


RygelTrackerVideoItem* rygel_tracker_video_item_construct (GType object_type, const char* id, const char* path, RygelTrackerContainer* parent, GError** error) {
	RygelTrackerVideoItem* self;
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);
	g_return_val_if_fail (parent != NULL, NULL);
	self = (RygelTrackerVideoItem*) rygel_tracker_item_construct (object_type, id, path, parent, error);
	return self;
}


RygelTrackerVideoItem* rygel_tracker_video_item_new (const char* id, const char* path, RygelTrackerContainer* parent, GError** error) {
	return rygel_tracker_video_item_construct (RYGEL_TYPE_TRACKER_VIDEO_ITEM, id, path, parent, error);
}


char** _dynamic_Get3 (DBusGProxy* self, const char* param1, const char* param2, char** param3, int param3_length1, int* result_length1, GError** error) {
	char** result;
	dbus_g_proxy_call (self, "Get", error, G_TYPE_STRING, param1, G_TYPE_STRING, param2, G_TYPE_STRV, param3, G_TYPE_INVALID, G_TYPE_STRV, &result, G_TYPE_INVALID);
	if (*error) {
		return NULL;
	}
	*result_length1 = g_strv_length (result);
	return result;
}


static void rygel_tracker_video_item_real_fetch_metadata (RygelTrackerItem* base, GError** error) {
	RygelTrackerVideoItem * self;
	GError * inner_error;
	char** _tmp0;
	gint keys_size;
	gint keys_length1;
	char** keys;
	char* _tmp1;
	char* _tmp2;
	char* _tmp3;
	char* _tmp4;
	char* _tmp5;
	char* _tmp6;
	char* _tmp7;
	char* _tmp8;
	char** _tmp9;
	gint values_size;
	gint values_length1;
	char** values;
	char** _tmp12;
	gint _tmp11_size;
	gint _tmp11_length1;
	gint _tmp10;
	char** _tmp11;
	char** _tmp13;
	char* _tmp18;
	char* _tmp20;
	const char* _tmp19;
	char* _tmp22;
	const char* _tmp21;
	char* _tmp23;
	self = (RygelTrackerVideoItem*) base;
	inner_error = NULL;
	_tmp0 = NULL;
	keys = (_tmp0 = g_new0 (char*, RYGEL_TRACKER_VIDEO_ITEM_METADATA_LAST_KEY + 1), keys_length1 = RYGEL_TRACKER_VIDEO_ITEM_METADATA_LAST_KEY, keys_size = keys_length1, _tmp0);
	_tmp1 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME] = (_tmp1 = g_strdup ("File:Name"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME]), NULL), _tmp1);
	_tmp2 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME] = (_tmp2 = g_strdup ("File:Mime"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME]), NULL), _tmp2);
	_tmp3 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE] = (_tmp3 = g_strdup ("File:Size"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE]), NULL), _tmp3);
	_tmp4 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE] = (_tmp4 = g_strdup ("Video:Title"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE]), NULL), _tmp4);
	_tmp5 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR] = (_tmp5 = g_strdup ("Video:Author"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR]), NULL), _tmp5);
	_tmp6 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH] = (_tmp6 = g_strdup ("Video:Width"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH]), NULL), _tmp6);
	_tmp7 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT] = (_tmp7 = g_strdup ("Video:Height"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT]), NULL), _tmp7);
	_tmp8 = NULL;
	keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE] = (_tmp8 = g_strdup ("DC:Date"), keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE] = (g_free (keys[RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE]), NULL), _tmp8);
	_tmp9 = NULL;
	values = (_tmp9 = NULL, values_length1 = 0, values_size = values_length1, _tmp9);
	_tmp12 = NULL;
	_tmp11 = (_tmp12 = _dynamic_Get3 (rygel_tracker_container_metadata, ((RygelTrackerItem*) self)->parent->category, ((RygelTrackerItem*) self)->path, keys, keys_length1, &_tmp10, &inner_error), _tmp11_length1 = _tmp10, _tmp11_size = _tmp11_length1, _tmp12);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
		values = (_vala_array_free (values, values_length1, (GDestroyNotify) g_free), NULL);
		return;
	}
	/* TODO: make this async */
	_tmp13 = NULL;
	values = (_tmp13 = _tmp11, values = (_vala_array_free (values, values_length1, (GDestroyNotify) g_free), NULL), values_length1 = _tmp11_length1, values_size = values_length1, _tmp13);
	if (_vala_strcmp0 (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE], "") != 0) {
		char* _tmp15;
		const char* _tmp14;
		_tmp15 = NULL;
		_tmp14 = NULL;
		((RygelMediaObject*) self)->title = (_tmp15 = (_tmp14 = values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_TITLE], (_tmp14 == NULL) ? NULL : g_strdup (_tmp14)), ((RygelMediaObject*) self)->title = (g_free (((RygelMediaObject*) self)->title), NULL), _tmp15);
	} else {
		char* _tmp17;
		const char* _tmp16;
		_tmp17 = NULL;
		_tmp16 = NULL;
		((RygelMediaObject*) self)->title = (_tmp17 = (_tmp16 = values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_FILE_NAME], (_tmp16 == NULL) ? NULL : g_strdup (_tmp16)), ((RygelMediaObject*) self)->title = (g_free (((RygelMediaObject*) self)->title), NULL), _tmp17);
	}
	if (_vala_strcmp0 (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE], "") != 0) {
		((RygelMediaItem*) self)->res.size = (glong) atoi (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_SIZE]);
	}
	if (_vala_strcmp0 (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH], "") != 0) {
		((RygelMediaItem*) self)->res.width = atoi (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_WIDTH]);
	}
	if (_vala_strcmp0 (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT], "") != 0) {
		((RygelMediaItem*) self)->res.height = atoi (values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_HEIGHT]);
	}
	_tmp18 = NULL;
	((RygelMediaItem*) self)->date = (_tmp18 = rygel_tracker_item_seconds_to_iso8601 ((RygelTrackerItem*) self, values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_DATE]), ((RygelMediaItem*) self)->date = (g_free (((RygelMediaItem*) self)->date), NULL), _tmp18);
	_tmp20 = NULL;
	_tmp19 = NULL;
	((RygelMediaItem*) self)->res.mime_type = (_tmp20 = (_tmp19 = values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_MIME], (_tmp19 == NULL) ? NULL : g_strdup (_tmp19)), ((RygelMediaItem*) self)->res.mime_type = (g_free (((RygelMediaItem*) self)->res.mime_type), NULL), _tmp20);
	_tmp22 = NULL;
	_tmp21 = NULL;
	((RygelMediaItem*) self)->author = (_tmp22 = (_tmp21 = values[RYGEL_TRACKER_VIDEO_ITEM_METADATA_AUTHOR], (_tmp21 == NULL) ? NULL : g_strdup (_tmp21)), ((RygelMediaItem*) self)->author = (g_free (((RygelMediaItem*) self)->author), NULL), _tmp22);
	_tmp23 = NULL;
	((RygelMediaItem*) self)->res.uri = (_tmp23 = rygel_tracker_item_uri_from_path ((RygelTrackerItem*) self, ((RygelTrackerItem*) self)->path), ((RygelMediaItem*) self)->res.uri = (g_free (((RygelMediaItem*) self)->res.uri), NULL), _tmp23);
	keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
	values = (_vala_array_free (values, values_length1, (GDestroyNotify) g_free), NULL);
}


static void rygel_tracker_video_item_class_init (RygelTrackerVideoItemClass * klass) {
	rygel_tracker_video_item_parent_class = g_type_class_peek_parent (klass);
	RYGEL_TRACKER_ITEM_CLASS (klass)->fetch_metadata = rygel_tracker_video_item_real_fetch_metadata;
}


static void rygel_tracker_video_item_instance_init (RygelTrackerVideoItem * self) {
}


GType rygel_tracker_video_item_get_type (void) {
	static GType rygel_tracker_video_item_type_id = 0;
	if (rygel_tracker_video_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (RygelTrackerVideoItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rygel_tracker_video_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RygelTrackerVideoItem), 0, (GInstanceInitFunc) rygel_tracker_video_item_instance_init, NULL };
		rygel_tracker_video_item_type_id = g_type_register_static (RYGEL_TYPE_TRACKER_ITEM, "RygelTrackerVideoItem", &g_define_type_info, 0);
	}
	return rygel_tracker_video_item_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




